# This file is part of MUST (Marmot Umpire Scalable Tool)
#
# Copyright (C)
#   2011-2014 ZIH, Technische Universitaet Dresden, Federal Republic of Germany
#   2011-2014 Lawrence Livermore National Laboratories, United States of America
#   2013-2014 RWTH Aachen University, Federal Republic of Germany
#
# See the file LICENSE.txt in the package base directory for details
#
# @file CMakeLists.cmake
#       CMake file for the modules/Common directory.
#
# @author Tobias Hilbrich
# @date 04.02.2011

###Run all feature tests that influence config.h
#Optional Fortran datatypes
IF (GTI_ENABLE_FORTRAN)
      SET (OptionalFortranTypes 
                                MPI_DOUBLE_COMPLEX 
                                MPI_REAL2 MPI_REAL4 MPI_REAL8 MPI_REAL16
                                MPI_INTEGER1 MPI_INTEGER2 MPI_INTEGER4 MPI_INTEGER8 MPI_INTEGER16
                                MPI_COMPLEX8 MPI_COMPLEX16 MPI_COMPLEX32
                                MPI_LOGICAL1 MPI_LOGICAL2 MPI_LOGICAL4 MPI_LOGICAL8 MPI_LOGICAL16
                                MPI_2COMPLEX
                                MPI_2DOUBLE_COMPLEX
                                CACHE INTERNAL "All the optional Fortran types"
                              )
  
      FOREACH (type ${OptionalFortranTypes})
          STRING (TOLOWER ${type} type_lower)
          featureTestMpi ( "ft_${type_lower}.f" Fortran HAVE_${type})
          
          #Set extra variable used in Fortran source for datatype handling
          IF (HAVE_${type})
              SET (F_${type} "${type}" CACHE INTERNAL "")
          ELSE (HAVE_${type})
              SET (F_${type} "MPI_DATATYPE_NULL" CACHE INTERNAL "")
          ENDIF (HAVE_${type})
      ENDFOREACH (type)
ENDIF (GTI_ENABLE_FORTRAN)
  
#Optional datatypes C
featureTestMpi ("ft_mpi_long_long_int.c" C HAVE_MPI_LONG_LONG_INT)
featureTestMpi ("ft_mpi_long_long.c" C HAVE_MPI_LONG_LONG)
featureTestMpi ("ft_mpi_unsigned_long_long.c" C HAVE_MPI_UNSIGNED_LONG_LONG)
featureTestMpi ("ft_mpi_wchar.c" C HAVE_MPI_WCHAR)
featureTestMpi ("ft_mpi_signed_char.c" C HAVE_MPI_SIGNED_CHAR)

#MPI-2 functions
SET (MUST_SUPPORTED_MPI2_FUNCS
    MPI_Get_address
    MPI_Type_get_extent
    MPI_Type_get_true_extent
    MPI_Type_create_hvector
    MPI_Type_create_hindexed
    MPI_Type_create_struct
    MPI_Type_create_resized
    MPI_Type_create_indexed_block
    MPI_Type_create_subarray
    MPI_Type_create_darray
    MPI_Reduce_scatter
    MPI_Reduce_scatter_block
    MPI_Exscan
    MPI_Alltoallw
    CACHE INTERNAL "All the mpi-2 datatype functions"
)
FOREACH (func ${MUST_SUPPORTED_MPI2_FUNCS})
    STRING (TOLOWER ${func} testName)
    STRING (TOUPPER ${func} haveName)
    SET (testName "ft_${testName}.cpp")
    SET (haveName "HAVE_${haveName}")

    featureTestMpi ("${testName}" CXX ${haveName})
    IF (${haveName})
        SET (${haveName}_POSTFIX "")
        SET (${haveName}_PREFIX "")
    ELSE (${haveName})
        SET (${haveName}_POSTFIX "-->")
        SET (${haveName}_PREFIX "<!--")
    ENDIF (${haveName})

ENDFOREACH (func)

#MPI-2 constants
featureTestMpi ("ft_mpi_distribute_block.c" C HAVE_MPI_DISTRIBUTE_BLOCK)
featureTestMpi ("ft_mpi_order_c.c" C HAVE_MPI_ORDER_C)
featureTestMpi ("ft_mpi_status_ignore.c" C HAVE_MPI_STATUS_IGNORE)
featureTestMpi ("ft_mpi_statuses_ignore.c" C HAVE_MPI_STATUSES_IGNORE)

# handle convert macros may be present
SET (HandleConvertMacros
    MPI_COMM_C2F
    MPI_ERRHANDLER_C2F
    MPI_FILE_C2F
    MPI_GROUP_C2F
    MPI_INFO_C2F
    MPI_OP_C2F
    MPI_REQUEST_C2F
    MPI_STATUS_C2F
    MPI_TYPE_C2F
    MPI_WIN_C2F
    CACHE INTERNAL "All the handle convert macros"
)

FOREACH (type ${HandleConvertMacros})
    STRING (TOLOWER ${type} type_lower)
    featureTestMpi ( "ft_${type_lower}2c.cpp" CXX HAVE_${type})
ENDFOREACH (type)

# TODO: test&compare

##Set the types to use for the converted MPI handles
SET (DATATYPE_F_TYPE "uint64_t" CACHE INTERNAL "Return type of MPI_Datatype_c2f.")
SET (COMM_F_TYPE "uint64_t" CACHE INTERNAL "Return type of MPI_Comm_c2f.")
SET (REQUEST_F_TYPE "uint64_t" CACHE INTERNAL "Return type of MPI_Request_c2f.")
SET (OP_F_TYPE "uint64_t" CACHE INTERNAL "Return type of MPI_Op_c2f.")
SET (GROUP_F_TYPE "uint64_t" CACHE INTERNAL "Return type of MPI_Group_c2f.")
SET (ERRHANDLER_F_TYPE "uint64_t" CACHE INTERNAL "Return type of MPI_Errhandler_c2f.")
SET (KEYVAL_TYPE "int" CACHE INTERNAL "Type to use for keyvals.")

SET (LOGICAL_TS_TYPE "unsigned long" CACHE INTERNAL "Type to use for MUST's logical time stamps.")

#Type used for MPI_Aint 
SET (AINT_TYPE "int64_t" CACHE INTERNAL "Type that is equal to MPI_Aint.")
