/* This file is part of MUST (Marmot Umpire Scalable Tool)
 *
 * Copyright (C)
 *  2011-2014 ZIH, Technische Universitaet Dresden, Federal Republic of Germany
 *  2011-2014 Lawrence Livermore National Laboratories, United States of America
 *  2013-2014 RWTH Aachen University, Federal Republic of Germany
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file QOpCommunication.h
 *       @see must::QOpCommunication.
 *
 *  @date 28.02.2013
 *  @author Tobias Hilbrich
 */

#ifndef QOPCOMMUNICATION_H
#define QOPCOMMUNICATION_H

#include "MustEnums.h"
#include "BaseIds.h"
#include "MustTypes.h"
#include "I_CommTrack.h"

#include "QOp.h"

using namespace gti;

namespace must
{
    /**
     * Any communication operation.
     */
    class QOpCommunication : public QOp
    {
    public:

        /**
         * Constructor for any communication operation.
         * @see QOp::QOp
         * @param comm communicator that will be managed by this
         *              operation. I.e. it will free this communicator
         *              in its destructor.
         */
        QOpCommunication (
                DWaitState *dws,
                MustParallelId pId,
                MustLocationId lId,
                MustLTimeStamp ts,
                I_CommPersistent *comm
        );

        /**
         * Returns communicator used by this communication operation.
         * @return comm.
         */
        I_Comm* getComm (void);

        /**
         * @see QOp::printVariablesAsLabelString
         */
        virtual std::string printVariablesAsLabelString (void);

        /**
         * @see QOp::getUsedComms
         */
        virtual std::list<I_Comm*> getUsedComms (void);

    protected:
        I_CommPersistent *myComm;

        /**
         * Destructor.
         */
        virtual ~QOpCommunication (void);
    };

} /*namespace must*/

#endif /*QOPCOMMUNICATION_H*/
