/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2013
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2013
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file sendProcNullNoDl.c
 * Simple send-recv test that is complicated by the usage of MPI_PROC_NULL.
 *
 * Description:
 * Matching and correct MPI calls (NoError).
 *
 * @author Tobias Hilbrich
 */

#include <mpi.h>
#include <stdio.h>

int main (int argc, char** argv)
{
    int rank,size,buf1, buf2;
    MPI_Status status;

    MPI_Init(&argc,&argv);
    MPI_Comm_rank (MPI_COMM_WORLD, &rank);
    MPI_Comm_size (MPI_COMM_WORLD, &size);

    //Enough tasks ?
    	if (size < 2)
    	{
    		printf ("This test needs at least 2 processes!\n");
    		MPI_Finalize();
    		return 1;
    	}

    	//Say hello
    	printf ("Hello, I am rank %d of %d processes.\n", rank, size);

    	if (rank == 0)
    	{
    	    MPI_Send (&buf1, 1, MPI_INT, MPI_PROC_NULL, 666, MPI_COMM_WORLD);
    	    MPI_Recv (&buf2, 1, MPI_INT, 1, 666, MPI_COMM_WORLD, &status);
    		MPI_Send (&buf1, 1, MPI_INT, 1, 666, MPI_COMM_WORLD);
    	}

    	if (rank == 1)
    	{
    	    MPI_Send (&buf1, 1, MPI_INT, MPI_PROC_NULL, 666, MPI_COMM_WORLD);
    	    MPI_Send (&buf1, 1, MPI_INT, 0, 666, MPI_COMM_WORLD);
    		MPI_Recv (&buf2, 1, MPI_INT, 0, 666, MPI_COMM_WORLD, &status);
    	}

    	//Say bye bye
    	printf ("Signing off, rank %d.\n", rank);

    MPI_Finalize ();

    return 0;
}
