#! @BASH@
#
# MUST (Marmot Umpire Scalable Tool)
# Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2013
# Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2013
#
# See the file LICENSE.txt in the package base directory for details
#
#@@
# mustrun - tool to run applications with the MPI correctness tool MUST
#

#----------------- Function ----------------
#name: printHelp
#purpose: prints the help for this script
#-------------------------------------------
printHelp()
{
  if MYSTTY=$(which stty)
  then
      WIDTH=$($MYSTTY size | cut -d' ' -f2)
  elif MYTPUT=$(which tput)
  then 
      WIDTH=$($MYTPUT cols)
  else
      WIDTH=80
  fi
  if test $WIDTH -lt 80
  then
    WIDTH=80
  elif test $WIDTH -gt 130
  then
    WIDTH=130
  fi
  if ! MYFMT="$(which fmt) -w $WIDTH"
  then
      MYFMT="cat"
  fi
  (echo "\"mustrun\" from MUST v@MUST_VERSION@"
  echo "Prepares and runs an application that is linked with P^nMPI for runtime analysis" 
  echo "        with the MPI correctness tool MUST."
  echo ""
  echo "Replace your regular mpiexec/mpirun command with mustrun, usage:"
  echo ""
  echo "mustrun [--help|-help|--must:help] [--must:nocrash] [--must:layout <LAYOUT.xml>]" 
  echo "        [--must:mode {prepare|run|preparerun}] [--must:mpiexec <RUNCOMMAND>]" 
  echo "        [--must:np <NP-SWITCH>] [--must:temp <DIR>] [--must:apis <API-XMLS>]" 
  echo "        [--must:analyses <ANALYSES-XMLS>] [--must:verbose] [--must:quiet]"
  echo "        <MPIRUNARGS> <COMMAND> <ARGS>"
  echo ""
  (echo "--help,-help,--must:help:"
  echo "        Prints this information."
  echo "--must:nocrash:"
  echo "        When using the default layout, switches on asynchronous communication to"
  echo "        improve tool performance, only use this option if your applications"
  echo "        suffers from no crash."
  echo "--must:layout <LAYOUT.xml>:"
  echo "        Overrides the default MUST layout with the given layout."
  echo "--must:nodesize <NUMBER>:"
  echo "        Specifies the systems number of processes per core, required for"
  echo "        application crash handling. See the MUST manual for details on the"
  echo "        resulting layouts."
  echo "--must:distributed:"
  echo "        Distributed correctness checks, requires multiple layers of extra"
  echo "        processes."
  echo "--must:nodl:"
  echo "        Disables deadlock detection if --must:distributed is specified."
  echo "--must:fanin <NUMBER>:"
  echo "        Gives the maximal fanin for distributed layout. Default: 16. See"
  echo "        the MUST manual for details, automatically sets \"--must:distributed\"."
  echo "--must:fillnodes:"
  echo "        Forces MUST to fill all used nodes (helps for some batch systems)."
  echo "        Only use if necessary, requires --must:distributed and --must:nodesize."
  echo "--must:language {c|fortran}:"
  echo "        Specifies in which programming language your program is written. This is"
  echo "        needed to use the right library of pnmpi. Use \"c\" for C++ applications."
  echo "        If nothing is set, the correct language will be detected automatically."
  echo "--must:mode {prepare|run|preparerun}:"
  echo "        Operation mode of mustrun, \"prepare\" only creates the intermediate files"
  echo "        needed by MUST, \"run\" only runs an application that was previously"
  echo "        prepared with mustrun, \"preparerun\" performs a \"prepare\" first and a"
  echo "        \"run\" afterwards, default is \"preparerun\"."
  echo "--must:info:"
  echo "        Print the size for the (explicit or implicit) specified layout."
  echo "--must:np <NP-SWITCH>:"
  echo "        Tells MUST to search for <NP-SWITCH> in <MPIRUNARGS> for determining the"
  echo "        number of application processes, example: \"--must:np -np\"."
  echo "--must:mpiexec <RUNCOMMAND>:"
  echo "        Tells MUST to use the given command instead of mpiexec."
  echo "--must:temp <DIR>:"
  echo "        Specifies which directory should be used to store intermediate files, or"
  echo "        in which directory the preparation of this run was made, default is"
  echo "        \"must_temp\"."
  echo "--must:clean:"
  echo "        Removes all intermediate module and layout files generated by MUST after"
  echo "        the run completes. Does not remove MUSTs output files."
  echo "--must:cleanshm:"
  echo "        Removes shared memory fragments remaining after crash of application."
  echo "        Could raise problems when run multiple instances of mustrun on a host."
  echo "--must:analyses <ANALYSES-XMLS>:"
  echo "        Gives extra analysis specifications to the weaver, multiple xmls need to"
  echo "        be separated with a colon."
  echo "--must:apis <API-XMLS>:"
  echo "        Gives extra api specifications to the weaver, multiple xmls need to be"
  echo "        separated with a colon."
  echo "--must:verbose:"
  echo "        Prints extensive information, use this when submitting bug reports." 
  echo "--must:timing:"
  echo "        Adds a \"time\" to the actual mpirun call, used to discern mustrun"
  echo "        overheads from the actual runtime." 
  echo "--must:quiet:"
  echo "        Causes mustrun to give no output at all (if successful)."
  echo "--must:output {html|stdout}:"
  echo "        Specifies the output format of MUST correctness messages. Default is"
  echo "        \"html\" which provides a correctness report in a file named"
  echo "        \"MUST_Output.html\". Use \"stdout\" if you prefer to get MUSTs output"
  echo "        printed to std out. Note that MUST will usually print this output on an" 
  echo "        extra task, so it may be placed oddly."
  echo "--must:ddt:"
  echo "        Enables the MUST-DDT integration. Must adds ddt-client (or the value"
  echo "        of --must:precmdddt) in front of the binary to link the starting"
  echo "        application with a DDT instance in the manual launch mode."
  echo "        This disables default output types (html/stdout) and rather issues"
  echo "        a function that DDT traps with breakpoints."
  echo "--must:precmdddt:"
  echo "        Specifies the command that MUST puts in front of the binary to support"
  echo "        DDT's manual launch mode, default is \"ddt-client\"."
  echo "--must:capture:"
  echo "        Stores correctness messages of MUST in a format that allows MUST to"
  echo "        reproduce the messages in a latter run. Usage intention is for the"
  echo "        MUST-DDT integration to trigger breakpoints on calls that cause errors"
  echo "        (Especially for errors that MUST detects remotely)."
  echo "        Default is off, is automatically enabled if --must:ddt is present."
  echo "--must:reproduce:"
  echo "        Causes MUST to trigger error messages that have been recorded with"
  echo "        --must:capture in a previous run. Use this to trigger breakpoints"
  echo "        for non-local correctness errors in the MUST-DDT integration."
  echo "        IMPORTANT: only for applications that use the same"
  echo "                   deterministic MPI operations in all executions."
  echo "--must:pnmpi-linked:"
  echo "        tells mustrun that pnmpi is already linked to the executable, per"
  echo "        default MUST tries to detect this automatically, use this, to override"
  echo "        its decission."
  echo "<MPIRUNARGS>:"
  echo "        Arguments you usually pass to mpirun/mpiexec, mustrun searches for a"
  echo "        \"-n\" \"-np\" \"-p\" in the given order to determine how many tasks"
  echo "        should be used for the application."
  echo "<COMMAND>:"
  echo "        Application to execute."
  echo "<ARGS>: Application arguments." 
  echo ""
  echo "Note that the default layout used for MUST requires one additional process," 
  echo "mustrun will execute an mpiexec with the number of application processes you" 
  echo "specified plus 1, make sure to allocate the sufficient resources."
  echo ""
  echo "Example: mustrun -np 4 myapp.exe"
  echo "(Prepares and runs \"maypp.exe\" with default layout)"
  echo ""
  echo "Contact: tobias.hilbrich@tu-dresden.de"
  echo ""
  for i in "$@"
  do
    echo "$i"
  done) | $MYFMT )>&2
  exit 1
}

#----------------- Function ----------------
#name: printVersions
#purpose: generates a prefix for echo output
#-------------------------------------------

printVersions()
{
  echo "MUST v@MUST_VERSION@"
  echo "based on"
  echo "GTI v@GTI_VERSION@"
  echo "and"
  echo "PnMPI v@PNMPI_VERSION@"
}

#----------------- Function ----------------
#name: mustecho
#purpose: generates a prefix for echo output
#-------------------------------------------
NO_ECHO_NL=0
mustecho()
{
    # if this is not a continued echo
    if [ $NO_ECHO_NL -ne 1 ]
    then
        # prefix
        echo -n "[MUST] "
    fi
    NO_ECHO_NL=0
    # record echo to be continued
    OPTIND=1
    if getopts "n" opt
    then
        NO_ECHO_NL=1
    fi
    echo "$@"
}

#----------------- Function ----------------
#name: musterror
#purpose: generates a prefix for error output
#-------------------------------------------
musterror()
{
    # if there is a continuing echo
    if [ $NO_ECHO_NL -eq 1 ]
    then
        echo
    fi
    # prefix
    echo -n "[MUST-ERROR] " 
    echo "$@"
}

#----------------- Function ----------------
#name: remove_single_opt
#purpose: removes a must option from input
#-------------------------------------------
remove_single_opt()
{
    input=$(echo "$input" | sed -e "s/--must:$1//")
}

#----------------- Function ----------------
# name: get_opt_arg(option)
# purpose: get the argument associated with --must:option
# return: argument via stdout
#-------------------------------------------
get_opt_arg()
{
    local ret=$(echo "$input" | sed -e "s/.*--must:$1[[:space:]]\([^[:space:]]\{1,\}\).*/\1/")
    # found an argument?
    if test "$ret" == "$input"
    then
        printHelp "Missing argument for option --must:$opt"
    fi
    # argument should not start with '-'
    if [[ "$1" != "np" && "$ret" == -* ]]
    then
        printHelp "Missing argument for option --must:$opt"
    fi
    echo $ret
}

#----------------- Function ----------------
#name: remove_opt_and_arg
#purpose: removes a must option and argument from input
#-------------------------------------------
remove_opt_and_arg()
{
    input=$(echo "$input" | sed -e "s/--must:$1[[:space:]][^[:space:]]\{1,\}//")
}

#----------------- Function ----------------
#name: check_opt_arg
# args: ($1=option, $2=argument)
#purpose: validate option arguments
#-------------------------------------------
check_opt_arg()
{
    case $1 in
    nodesize)
        nodesizearg="argument nodesize $2"
        ;;
    layout)
        absolut_path arg
        if [ ! -f "$2" ]
        then
            musterror "Can't find the given layout file \""$2"\"!"
            exit 1
        fi
        ;;
    mode)
        case "$2" in run | preparerun | prepare | auto);; #ok, do nothing
        *) # unknown
            printHelp "Unknown argument for option --must:mode : $2" \
                        "                                         (valid: run | prepare | preparerun)"
            ;;
        esac
        ;;
    language)
        case "$2" in c | fortran);; #ok, do nothing
        *) # unknown
            printHelp "Unknown argument for option --must:language : $2" \
                        "                                             (valid: c | fortran)"
            ;;
        esac
        ;;
    output)
        case "$2" in html | stdout);; #ok, do nothing
        *) # unknown
            printHelp "Unknown argument for option --must:output : $2" \
                        "                                           (valid: html | stdout)"
        esac
        ;;
    fanin)
        fanin_given=1
        ;;
    esac
}

#----------------- Function ----------------
#name: absolut_path(path-variable-name)
#purpose: makes a path absolut
#-------------------------------------------
absolut_path()
{
    local result=$1
    eval local val=\$$1
    if [[ "$val" != /* ]]
    then
        val=$PWD/$val
    fi
    eval $result="'$val'"
}

#----------------- Function ----------------
#name: get_num_procs
#purpose: get the number of procs given on commandline (e.g.: -np [n])
#return: number of procs via stdout
#-------------------------------------------
get_num_procs()
{
    local switch
    for switch in $np_arg
    do
      local pTemp=`echo " $input" | sed -e "s/.*[[:space:]]${switch}[[:space:]]\([^[:space:]]\{1,\}\).*/\1/"`
      
      if [ "$pTemp" != " $input" ]
      then
          p=$pTemp
          switchToUse=$switch
          return 0
      fi
    done
    return 1
}

#----------------- Function ----------------
# name: get_checksum()
# purpose: get a checksum for run relevant arguments
# return: argument via stdout
#-------------------------------------------
get_checksum()
{
	if [ "x@MD5SUM_FOUND@" == "xTRUE" ]
	then
        md5sum="@MD5SUM@"
    else
        md5sum="cat"
    fi
    (
        # number of processes
        if ! get_num_procs
        then
            # error, not re-use build
            date
        else
            echo $p
        fi
        # explicit layout-file or layoutmode
        if [ -n "${layout_arg}" ];
        then
            cat ${layout_arg}
        else
            echo $layoutMode
        fi
        ls -l $temp_arg/BUILD/install/modules/
        ls -l @CMAKE_INSTALL_PREFIX@/modules/
        ls -l @GTI_MODULE_DIR@/
        echo ${nodesize_arg} ${fanin_arg} ${firstfanin_arg} ${do_dl} ${do_reproduce} ${do_ddt} ${do_capture}
        # environment variables
        echo $oldenv
    ) | $md5sum
}


#----------------- Function ----------------
#name: gen_layout
#purpose: tests the given layout or generates a new one
#-------------------------------------------
gen_layout()
{
#####################################################################
## 2) What layout are we using (--must:layout)

if [ -n "${layout_arg}" ];
then
    layoutFile=$layout_arg
    printLayoutFile=$layout_arg
else
    layoutFile=layout.xml
    printLayoutFile=${temp_arg}/$layoutFile
    if ! get_num_procs
    then
        musterror "Could not determine number of MPI tasks, use \"--must:np <NP-SWITCH>\" to tip me."
        exit 1
    fi

    strategy="CStratThreadedUp" ## Strategy between layer 0 and 1
	strategyDefault="CStratThreadedUp" ## Strategy between other layers
	intraStrategy="CStratThreadedIntra"
    place="mpi_place"

    if [ $layoutMode -eq 1 ]
    then
      strategy="CStratSimpleUp"
	  strategyDefault="CStratSimpleUp"
    fi
	
	if [ $do_distributed -eq 1 ]
	then
	  if [ $do_dl -eq 1 ]	
	  then
		strategy="CStratPRecvUp"
     	strategyDefault="CStratPRecvUp"
		intraStrategy="CStratPRecvIntra"
	  fi
	fi
	
	#Set whether we need a case for level 1 (for distributed layouts with more than one layer)
	layer1CaseLabel=999
	if [ $layoutNumLayers -gt 2 ]
	then
		layer1CaseLabel=1	
	fi
	
    cat <<End-of-File > $printLayoutFile
<layout-specification>
  <levels>
End-of-File
    for (( i=0; i <= $[$layoutNumLayers-1]; i++))
    do
        echo >> $printLayoutFile "<level order=\"$i\" size=\"${layoutLayerSizes[$i]}\" place-name=\"$place\"><analyses>"
        case $i in
# application level analyses
        0)
			#We have a "reproduce" errors mode in which we don't run checks, but rather read a previoud error log instead
			#So if we aren't doing the reproduce mode, we apply checks, and otherwise we apply the error reproducer
			if [ $do_reproduce -eq 0 ]
			then
				cat <<End-of-File >> $printLayoutFile
					<analysis name="DatatypePredefs" group="MUST_Resource"/>
					<analysis name="CommPredefs" group="MUST_Resource"/>
					<analysis name="FinalizeNotify" group="MUST_Base"/>
					<analysis name="DCollectiveInitNotify" group="MUST_DistributedDeadlock"/>
					<analysis name="OpSplitter" group="MUST_Deadlock"/>
					<analysis name="WcUpdate" group="MUST_Deadlock"/>
					<analysis name="IntegerChecks" group="MUST_BasicChecks"/>
					<analysis name="BasicIntegrities" group="MUST_BasicChecks"/>
					<analysis name="OperationChecks" group="MUST_BasicChecks"/>
					<analysis name="CommChecks" group="MUST_CommChecks"/>
					<analysis name="RequestChecks" group="MUST_RequestChecks"/>
					<analysis name="BufferChecks" group="MUST_BufferChecks"/>
					<analysis name="DatatypeChecks" group="MUST_DatatypeChecks"/>
					<analysis name="OverlapChecks" group="MUST_OverlapChecks"/>
					<analysis name="GroupChecks" group="MUST_GroupChecks"/>
					<analysis name="LeakChecks" group="MUST_LeakChecks"/>
					<analysis name="RequestCondition" group="MUST_RequestCondition"/>
					<analysis name="CollectiveCondition" group="MUST_CollectiveCondition"/>
					<analysis name="OnlyOnRootCondition" group="MUST_OnlyOnRootCondition"/>
					<analysis name="CompletionCondition" group="MUST_CompletionCondition"/>
End-of-File
            else 
			    #We do reproduce errors, so add the reproducer module
				echo >> $printLayoutFile "<analysis name=\"MessageReproducer\" group=\"MUST_Base\"/>"
			fi
			
			# Add DDT Logger if we run with DDT (irrespective of whether we reproduce or not)
			if [ $do_ddt -eq 1 ]
            then
                echo >> $printLayoutFile "<analysis name=\"MsgLoggerDdt\" group=\"MUST_Base\"/>"
            fi
        ;;
# first level analyses
         $layer1CaseLabel)
			if [ $do_reproduce -eq 0 ]
			then
				# Distributed layouts need either DP2PMatch or DWaitState (the latter drags in p2p-matching) on first non-application layer
				if [ $layoutMode -gt 3 ]
				then
					if [ $do_dl -eq 1 ]
					then
					echo >> $printLayoutFile "<analysis name=\"DWaitState\" group=\"MUST_DistributedDeadlock\"/>" 
					else
					echo >> $printLayoutFile "<analysis name=\"DP2PMatch\" group=\"MUST_DistributedDeadlock\"/>" 
					fi
				fi
			fi #reproduce ?
         ;;
# root level analyses
        $[$layoutNumLayers-1])
			if [ $do_reproduce -eq 0 ]
			then
				if [ $layoutMode -lt 4 ]
				then
					#centralized modes (1-3) use blocking state (DL + P2P + Coll)
					echo >> $printLayoutFile "<analysis name=\"BlockingState\" group=\"MUST_Deadlock\"/>"
				else
					#distributed modes (4-5) use distributed collective match 
					echo >> $printLayoutFile "<analysis name=\"DCollectiveMatchRoot\" group=\"MUST_DistributedDeadlock\"/>"
				
					#If we do deadlock detection, we need the central wait state modules
					if [ $do_dl -eq 1 ]
					then
						echo >> $printLayoutFile "<analysis name=\"DWaitStateCollMgr\" group=\"MUST_DistributedDeadlock\"/>"
						echo >> $printLayoutFile "<analysis name=\"DWaitStateWfgMgr\" group=\"MUST_DistributedDeadlock\"/>"
					fi
								
					#we need to add our layer 1 modules if we only have one layer
					if [ "x$layoutNumLayers" == "x2" ]
					then
						if [ $do_dl -eq 1 ]
						then
							echo >> $printLayoutFile "<analysis name=\"DWaitState\" group=\"MUST_DistributedDeadlock\"/>" 
						else
							echo >> $printLayoutFile "<analysis name=\"DP2PMatch\" group=\"MUST_DistributedDeadlock\"/>" 
						fi
					fi
				fi
			fi

            # Logger base selection: StdOut | DDT+Reproduce | HTML
            if [ "$output_arg" == "stdout" ]
            then
                echo >> $printLayoutFile "<analysis name=\"MsgLoggerStdOut\" group=\"MUST_Base\"/>"
            elif [ $do_ddt -eq 1 ]
            then
				#For DDT, only add the logger modules if we aren't reproducing, otherwise the application layer DDT Logger module does the complete job
				if [ $do_reproduce -eq 0 ]
				then
					echo >> $printLayoutFile "<analysis name=\"MsgLoggerDdt\" group=\"MUST_Base\"/>"
					do_capture=1
				fi
            else
                echo >> $printLayoutFile "<analysis name=\"MsgLoggerHtml\" group=\"MUST_Base\"/>"
            fi
			
			# Logger addition: ScoreP Format
            if [ $do_scorep -eq 1 ]
            then
                # for --must:scorep insert the scorep logger:
                echo >> $printLayoutFile "<analysis name=\"MsgLoggerScoreP\" group=\"MUST_Base\"/>"
            fi
			
			# Logger addition: ScoreP Format
            if [ $do_capture -eq 1 ]
            then
                # for --must:scorep insert the scorep logger:
                echo >> $printLayoutFile "<analysis name=\"MsgLoggerReproduce\" group=\"MUST_Base\"/>"
            fi
								
        ;;
# other level analyses
#         *)
#         ;;
        esac
    echo >> $printLayoutFile "</analyses></level>"
    done
        cat <<End-of-File >> $printLayoutFile    
  </levels>
  <communications>
    <default>
      <comm-strategy name="$strategyDefault">
        <settings></settings>
      </comm-strategy>
      <comm-protocol name="CProtMpiSplited">
        <settings></settings>
      </comm-protocol>
    </default>
    <connection top-level="1" bottom-level="0"
End-of-File
    if [ -n "${nodesize_arg}" ]
    then
        cat <<End-of-File >> $printLayoutFile 
    distribution="by-block"
        blocksize="$[${nodesize_arg}-1]">
        <comm-strategy name="CStratIsendUp">
            <settings></settings>
          </comm-strategy>
          <comm-protocol name="CProtSharedMemory">
            <settings></settings>
          </comm-protocol
End-of-File
    else
        cat <<End-of-File >> $printLayoutFile 
>
        <comm-strategy name="$strategy">
            <settings></settings>
          </comm-strategy>
          <comm-protocol name="CProtMpiSplited">
            <settings></settings>
          </comm-protocol
End-of-File
    fi
    echo >> $printLayoutFile "></connection>"

    for (( i=2; i <= $[$layoutNumLayers-1]; i++ ))
    do
        echo >>$printLayoutFile "<connection top-level=\"$i\" bottom-level=\"$[$i-1]\"></connection>"
    done
	
	# Distributed modes need intra-layer connection on first non-application layer
	if [ $layoutMode -gt 3 ]
	then
        cat <<End-of-File >> $printLayoutFile 
	<connection top-level="1" bottom-level="1">
		<comm-strategy name="$intraStrategy">
			<settings></settings>
		</comm-strategy>
		<comm-protocol name="CProtMpiSplited">
			<settings></settings>
		</comm-protocol>
	</connection>
End-of-File
	fi

    echo >> $printLayoutFile "</communications></layout-specification>"
fi
if [ $do_verbose -eq 1 ]
then
    mustecho "Using layout file: $printLayoutFile"
fi

layoutSize=0
for j in $(grep '\bsize' $printLayoutFile| sed -e 's|.*size="\([^"]*\)".*|\1|')
do 
        layoutSize=$[$layoutSize+$j]
done

cd $workdir

}

#----------------- Function ----------------
#name: run_codegen
#purpose: generates the code for the given layout
#-------------------------------------------
run_codegen()
{
#####################################################################
## 3) Run the weaver and code generators

if [ $do_verbose -eq 1 ]
then
  mustecho "========= Step 2) Generation ========="
fi

#change to temp dir
cd $temp_arg

#MPI_Finalize is in an extra specification, if we do not do distribtued deadlock detection,
#it is finalizing the tool, if we do distributed deadlock detection it is not finalizing the
#tool; In that case DWaitState will issue the tool finalization
mpiFinSpec=@CMAKE_INSTALL_PREFIX@/specifications/mpi_specification_finalize_fin.xml

if [ $do_distributed -eq 1 -a $do_reproduce -eq 0 ]
then
  if [ $do_dl -eq 1 ]	
  then
	mpiFinSpec=@CMAKE_INSTALL_PREFIX@/specifications/mpi_specification_finalize_nofin.xml
  fi
fi

#run the weaver
set +e
if ! @WEAVER_EXE@ \
  $layoutFile \
  @GTI_SPECIFICATION@ \
  @CMAKE_INSTALL_PREFIX@/specifications/mpi_specification.xml \
  $mpiFinSpec \
  @CMAKE_INSTALL_PREFIX@/specifications/mpi_2_specification.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_base_api_spec.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_resource_track_api.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_deadlock_detection_api.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/request_condition_api.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/collective_condition_api.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/onlyOnRoot_condition_api.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/completion_condition_api.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_distributed_deadlock_detection_api.xml \
  $apis \
  @CMAKE_INSTALL_PREFIX@/specifications/mpi_base_specification.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_base_specification.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_resource_track.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_basic_checks.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_buffer_checks.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_comm_checks.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_request_checks.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_datatype_checks.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_overlap_checks.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_group_checks.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_leak_checks.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/request_condition.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/collective_condition.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_deadlock_detection.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/onlyOnRoot_condition.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/completion_condition.xml \
  @CMAKE_INSTALL_PREFIX@/specifications/must_distributed_deadlock_detection.xml \
  $analyses > output.out 2>output.err
  #TODO: add XMLS for further checks
then
  musterror "Running the weaver failed with the following output:"
  cat output.out
  cat output.err
  exit 1
fi

if [ $do_verbose -eq 1 ]
then
  musterror "==> Running weaver:"
  cat output.out
  cat output.err
else
  if [ $do_quiet -ne 1 ]
  then
    mustecho "Weaver ... success"
  fi
fi
  
#run the Generators
export PATH=@GTI_HOME@/bin:$PATH

if ! bash weaver-run-generators.sh > output.out 2>output.err
then
  musterror "Code generation failed with the following output:"
  cat output.out
  cat output.err
  exit 1
fi

if [ $do_verbose -eq 1 ]
then
  mustecho "==> Generating source code:"
  cat output.out
  cat output.err
else
  if [ $do_quiet -ne 1 ]
  then
    mustecho "Code generation ... success"
  fi
fi

#run the build file generation


if ! @BUILDGEN_EXE@ weaver-buildgen.xml > output.out 2>output.err
then
  musterror "Build file generation failed with the following output:"
  cat output.out
  cat output.err
  exit 1
fi

if [ $do_verbose -eq 1 ]
then
  mustecho "==> Generating build files:"
  cat output.out
  cat output.err
else
  if [ $do_quiet -ne 1 ]
  then
    mustecho "Build file generation ... success"
  fi
fi  
set -e

#the cat is superfluous, but is used to remove a possible error exit from the grep, so do not remove it!
cd $workdir

}

#----------------- Function ----------------
#name: build_modules
#purpose: compiles modules from the generated code
#-------------------------------------------
build_modules()
{
#####################################################################
## 4) Build the intermediate modules

cd $temp_arg

if [ -d BUILD ]
then
  rm -rf BUILD
fi

mkdir BUILD
cd BUILD

set +e
## Configure with cmake
CC=@MUST_C_COMPILER@ CXX=@MUST_CXX_COMPILER@ FC=@MUST_Fortran_COMPILER@ "@CMAKE_COMMAND@" ../ -DCMAKE_LINKER=@CMAKE_LINKER@ -DCMAKE_SHARED_LINKER_FLAGS="@CMAKE_SHARED_LINKER_FLAGS@" -DCMAKE_SHARED_LINKER_FLAGS_DEBUG="@CMAKE_SHARED_LINKER_FLAGS_DEBUG@" -DCMAKE_SHARED_LINKER_FLAGS_RELEASE="@CMAKE_SHARED_LINKER_FLAGS_RELEASE@" -DCMAKE_C_FLAGS="@CMAKE_C_FLAGS@" -DCMAKE_C_FLAGS_DEBUG="@CMAKE_C_FLAGS_DEBUG@" -DCMAKE_C_FLAGS_RELEASE="@CMAKE_C_FLAGS_RELEASE@" -DCMAKE_CXX_FLAGS="@CMAKE_CXX_FLAGS@" -DCMAKE_CXX_FLAGS_DEBUG="@CMAKE_CXX_FLAGS_DEBUG@" -DCMAKE_CXX_FLAGS_RELEASE="@CMAKE_CXX_FLAGS_RELEASE@" -DCMAKE_BUILD_TYPE=@CMAKE_BUILD_TYPE@ -DCMAKE_INSTALL_PREFIX=$PWD/install > output.out 2>output.err
  
if [ $? -ne 0 ]
then
  musterror "Configuration of intermediate build failed with the following output:"
  cat output.out
  cat output.err
  exit 1
fi

if [ $do_verbose -eq 1 ]
then
  mustecho "==> Configuring intermediate build:"
  cat output.out
  cat output.err
else
  if [ $do_quiet -ne 1 ]
  then
    mustecho "Configuring intermediate build ... success"
  fi
fi

makej=8
if [ $makej -gt $layoutSize ]
then
    makej=$layoutSize
fi

## Make
if [ $do_verbose -ne 1 -a $do_quiet -ne 1 ]
then
    mustecho -n "Building intermediate sources ... "
fi

  
if ! make -j${makej} > output.out 2>output.err
then
  musterror "Building intermediate sources failed with the following output:"
  cat output.out
  cat output.err
  exit 1
fi

if [ $do_verbose -eq 1 ]
then
  musterror "==> Building intermediate sources:"
  cat output.out
  cat output.err
else
  if [ $do_quiet -ne 1 ]
  then
    mustecho "success"
  fi
fi

## Make install

if ! make -j${makej} install > output.out 2>output.err
then
  musterror "Installing intermediate modules failed with the following output:"
  cat output.out
  cat output.err
  exit 1
fi

if [ $do_verbose -eq 1 ]
then
  mustecho "==> Installing intermediate modules:"
  cat output.out
  cat output.err
else
  if [ $do_quiet -ne 1 ]
  then
    mustecho "Installing intermediate modules ... success"
  fi
fi

set -e
cd $workdir
}

#----------------- Function ----------------
#name: pnmpi_config
#purpose: generates a pnmpi.config for the given layout
#-------------------------------------------
pnmpi_config()
{
#####################################################################
## 5) Create the P^nMPI configuration file
cd $temp_arg

## Get information on the layers we had in the layout
numLayers=`grep "num-layers" weaver-layout-info.xml | sed -e s/.*num-layers=\"// -e s/[^0-9].*//`
totalSize=0

for (( i = 0 ; i < $numLayers ; i++ ))
do
  size[$i]=`grep "id=\"$i\"" weaver-layout-info.xml | sed -e s"/.*size=\"//" -e s"/[^0-9].*//"`
  to[$i]=`grep "id=\"$i\"" weaver-layout-info.xml | sed -e s"/[^>]*[>]//" -e s"/[<][/]layer[>]//" -e s"/[<]to[>]/ /" -e s"/[<][/]to[>]/ /"`
  
  totalSize=$(($totalSize+${size[$i]}))
done

if [ $totalSize -ne $layoutSize ]
then
  mustecho "totalSize != layoutSize ($totalSize != $layoutSize)"
  exit 1
fi


## Store the total size in an info file
echo "$totalSize" > mustrun.info

## Print the merge file
cat <<End-of-File > level_merge.xml
<level-merge>
<set>
End-of-File

for (( i = 0 ; i < $numLayers ; i++ ))
do
  echo -e "\t\t<order>$i</order>" >> level_merge.xml
done

cat <<End-of-File >> level_merge.xml
</set>
</level-merge>
End-of-File

## Run the module configurator
set +e

if ! @MODCONFGEN_EXE@ weaver-mod-conf-input.xml level_merge.xml > output.out 2>output.err
then
  musterror "Generating P^nMPI configuration failed:"
  cat output.out
  cat output.err
  exit 1
fi

if [ $do_verbose -eq 1 ]
then
  mustecho "==> Generating P^nMPI configuration:"
  cat output.out
  cat output.err
else
  if [ $do_quiet -ne 1 ]
  then
    mustecho "Generating P^nMPI configuration ... success"
  fi
fi
set -e

##Add configuration for split module to the beginning of the config file
if [ "@USE_CALLPATH@" == "ON" -o "@USE_CALLPATH@" == "On" -o "@USE_CALLPATH@" == "on" -o "@USE_CALLPATH@" == "TRUE" -o "@USE_CALLPATH@" == "True" -o "@USE_CALLPATH@" == "true" ]
then
  cat <<End-of-File > pnmpi.conf
module libcallpathModule
End-of-File
else
      echo -n "" > pnmpi.conf
fi

  if [ "@HAVE_MPI_STATUS_IGNORE@" == "TRUE" -o "@HAVE_MPI_STATUSES_IGNORE@" == "TRUE" ]
  then
      cat <<End-of-File >> pnmpi.conf
module libstatusWrapper
End-of-File
  else
      echo -n "" >> pnmpi.conf
  fi
  
  if [ "$errorgenenv" ]
then
cat <<End-of-File >> pnmpi.conf
module liberrorGen
End-of-File
fi
  
cat <<End-of-File >> pnmpi.conf
module libcProtMpiSplitComm
argument num_sets $numLayers
$nodesizearg
End-of-File

confOut="weaver-mod-conf"
for (( i = 0 ; i < $numLayers ; i++ ))
do
  echo "argument size_$i ${size[$i]}" >> pnmpi.conf
  echo "argument stack_$i level_$i" >> pnmpi.conf
  
  confOut=$confOut.$i
done

index=0
commId=1
echo -n > pnmpi.conf_temp
for (( i = 0 ; i < $numLayers ; i++ ))
do
  for t in ${to[$i]}
  do
    echo "argument mapping$(($index)) $i:$commId:$t" >> pnmpi.conf_temp
    echo "argument mapping$(($index+1)) $t:$commId:$i" >> pnmpi.conf_temp
    index=$(($index+2))
    commId=$(($commId+1))
  done
done

echo "argument num_mappings $index" >> pnmpi.conf
cat < pnmpi.conf_temp >> pnmpi.conf
echo "" >> pnmpi.conf

cat < $confOut >>pnmpi.conf

# append the renaming of MPI_COMM_WORLD to end of stack level 0
cat pnmpi.conf | sed -e 's/stack[[:space:]]level_1/module libcProtMpiCommRewrite\
stack level_1/' > pnmpi.conf2
mv pnmpi.conf2 pnmpi.conf

#TODO disabled for the moment, needs further testing
#if [ $do_scorep -eq 1 ]
#then
#    # for --must:scorep insert the scorep_adapter_mpi at begin of stack 0:
## Version Frank had localy
#    cat pnmpi.conf | sed -e 's/stack[[:space:]]level_0/stack level_0\nmodule libscorep_adapter_mpi_patch\nmodule libscorep_mpp_mpi_patch/'  > pnmpi.conf2
## Original version
#    cat pnmpi.conf | sed -e 's/stack[[:space:]]level_0/stack level_0\
#module libscorep_adapter_mpi/'  > pnmpi.conf2
#    mv pnmpi.conf2 pnmpi.conf
#fi

if [ $do_isp -eq 1 ]
then
    # for --must:isp insert the isp_prof module at begin of stack 0:
    cat pnmpi.conf | sed -e 's/stack[[:space:]]level_0/stack level_0\
module libisp_prof/'  > pnmpi.conf2
    #                                           at end of stack 0:
#     cat pnmpi.conf | sed -e 's/stack[[:space:]]level_1/module libisp_prof\
#stack level_1/'  > pnmpi.conf2
    mv pnmpi.conf2 pnmpi.conf
fi

cd $workdir
}


#####################################################################
## Prepare the must environment
#####################################################################
prepare()
{

#####################################################################
## 1) Prepare a temp directory

if [ -d $temp_arg ]
then
    if [ ${do_quiet} -ne 1 ]
    then
        mustecho "Information: overwritting old intermediate data in directory \""$temp_arg"\"!"
    fi
    # no re-run for broken prepare
    rm -f ${temp_arg}/lastrun
else
    mkdir -p $temp_arg
fi

#Print result
if [ ${do_verbose} -eq 1 ]
then
    mustecho "Using intermediate directory \""$temp_arg"\"."
fi

#####################################################################
## 2) What layout are we using (--must:layout)
gen_layout 

#Set runOnBackend Variable
export runOnBackend=
if [ $mode_arg == "preparerun" ]
then
  runOnBackend="$mpiexec_arg $switchToUse 1"
fi

#####################################################################
## 3) Run the weaver and code generators
run_codegen 

#####################################################################
## 4) Build the intermediate modules
build_modules

#####################################################################
## 5) Create the P^nMPI configuration file
pnmpi_config

#####################################################################
## 6) Save checksum of current run
get_checksum > ${temp_arg}/lastrun

} # end prepare()


#----------------- Function ----------------
#name: calclayers
#purpose: info mode
#-------------------------------------------
calclayers()
{
if [ -n "${layout_arg}" ]
then
    layoutTotalSize=0
    for j in $(grep '\bsize' $layout_arg| sed -e 's|.*size="\([^"]*\)".*|\1|')
    do 
            layoutTotalSize=$[$layoutTotalSize+$j]
    done
else
	#If "fanin" is set, automatically set distributed!
	if [ "x$fanin_given" == "x1" ]
	then
		do_distributed=1;
	fi

    layoutNumLayers=1
    if ! get_num_procs
    then
        musterror "Could not determine number of MPI tasks, use \"--must:np <NP-SWITCH>\" to tip me."
        exit 1
    fi
    layoutLayerSizes[0]=$p
    layoutTotalSize=$p
    if [ $do_distributed -ne 1 ]
    then
        if [ -n "$nodesize_arg" ]
        then
            # Mode 2
            layoutMode=2
            layoutLayerSizes[1]=$[($p-1)/($nodesize_arg-1)+1]
            layoutLayerSizes[2]=1
            layoutTotalSize=$[$layoutTotalSize+${layoutLayerSizes[1]}+1]
            layoutNumLayers=3
        else
            layoutMode=1
            if [ "x$do_nocrash" == "x1" ]
            then
                layoutMode=3
            fi
            # Mode 1+3
            layoutLayerSizes[1]=1
            layoutTotalSize=$[$layoutTotalSize+1]
            layoutNumLayers=2
        fi
    else
        layoutMode=5
        firstLayer=1
        if [ -n "$nodesize_arg" ]
        then
            # Extra layer for mode 4
            layoutMode=4
            layoutLayerSizes[1]=$[($p-1)/($nodesize_arg-1)+1]
            layoutTotalSize=$[$layoutTotalSize+${layoutLayerSizes[1]}]
            layoutNumLayers=2
            firstLayer=0
        fi
        # Mode 4+5
        while [ ${layoutLayerSizes[$[$layoutNumLayers-1]]} -gt 1 ]
        do
            useFanIn=$fanin_arg
            if [ "x$firstLayer" == "x1" ]
            then
              useFanIn=$firstfanin_arg
            fi
            firstLayer=0
            layoutLayerSizes[$layoutNumLayers]=$[(${layoutLayerSizes[$[$layoutNumLayers-1]]}-1)/$useFanIn+1]
            layoutTotalSize=$[$layoutTotalSize+${layoutLayerSizes[$layoutNumLayers]}]
            layoutNumLayers=$[$layoutNumLayers+1]
        done
		
		# IF we do distributed deadlock detection: 
		## we currently have bug in shutdown that requires us to not run DWaitState on the root, so we add an extra layer if needed
		if [ $do_dl -eq 1 ]
		then
			if [ $layoutNumLayers -eq 2 ]
			then
				layoutLayerSizes[$layoutNumLayers]=1
				layoutTotalSize=$[$layoutTotalSize+${layoutLayerSizes[$layoutNumLayers]}]
				layoutNumLayers=$[$layoutNumLayers+1]
			fi			
		fi
    fi
fi
}

#----------------- Function ----------------
#name: fillnodes
#purpose: to be called after calclayers if 
#         all used nodes should be filled
#-------------------------------------------
fillnodes()
{
	#Is the mode fitting?
	if [ "$layoutMode" -ne "4" ]
    then
		musterror "The --must:fillnodes option requires that --must:nodesize and --must:distributed (or --must:fanin is specified)."
		exit 1
	fi
	
	numNodes=$[($layoutTotalSize-1)/($nodesize_arg)+1]
	openSlots=$[$numNodes*$nodesize_arg-$layoutTotalSize]
	
	#We must not change #app procs
	minlayer=1
	
	#We must not change #1-layer procs if we do app crash handling
	if [ -n "$nodesize_arg" ]
	then
		minlayer=2
	fi
	
	## First try to fill remaining slots to existing layers (without exceeding the set fan-in)
	redo=1
	while [ $redo -gt 0 ]
	do
		redo=0
		for (( i=$layoutNumLayers-1; i > $minlayer; i-- ))
		do
			useFanIn=$fanin_arg
       	    		if [ "x$i" == "x2" ]
       	    		then
       	      			useFanIn=$firstfanin_arg
       	    		fi

			if [ $[${layoutLayerSizes[$i]}*$useFanIn] -gt ${layoutLayerSizes[$i-1]} ]
			then
				if [ ${layoutLayerSizes[$i-2]} -gt ${layoutLayerSizes[$i-1]} ]
				then
					if [ $openSlots -gt 0 ]
					then
						layoutLayerSizes[i-1]=$[${layoutLayerSizes[$i-1]}+1]
						openSlots=$[$openSlots-1]
						layoutTotalSize=$[$layoutTotalSize+1]
						redo=1
					fi
				fi
			fi
		done
	done
	
	## If we couldn't distribute all slots yet, add extra layers of one process each
	while [ $openSlots -gt 0 ]
	do
		layoutLayerSizes[$layoutNumLayers]=1
		layoutNumLayers=$[$layoutNumLayers+1]
		openSlots=$[$openSlots-1]
		layoutTotalSize=$[$layoutTotalSize+1]
	done	
}

#----------------- Function ----------------
#name: print_info
#purpose: print info
#-------------------------------------------
print_info()
{
    mustecho -n "MUST configuration ... "
    
    if [ -n "${layout_arg}" ]
    then
        mustecho " with given layout.xml"
    else
        if [ $layoutMode -lt 4 ]
        then
            mustecho -n "centralized checks"
        else
            mustecho -n "distributed checks"
        fi
        
        case $layoutMode in 
            1)
                mustecho " with fall-back application crash handling (very slow)"
            ;; 
            2)
                mustecho " with application crash handling"
            ;;
            3) 
                mustecho " without application crash handling"
            ;;
            4)
                mustecho " with application crash handling"
            ;;
            5) 
                mustecho " without application crash handling"
            ;;
        esac
    fi

    if [ $do_info -eq 1 ]
    then
        mustecho "Required total number of processes ... $layoutTotalSize"
        mustecho "Number of application processes ... ${layoutLayerSizes[0]}"
        mustecho "Number of tool processes ... $[$layoutTotalSize - ${layoutLayerSizes[0]}]"
        
        if [ -n "$nodesize_arg" ]
        then
            mustecho "Total number of required nodes ... $[($layoutTotalSize-1)/($nodesize_arg)+1]"				
        fi
        
        mustecho -n "Tool layers sizes ... "
        for (( i=0; i < $layoutNumLayers; i++ ))
        do
            if [ $i -gt 0 ]
            then
                mustecho -n ":"
            fi
            mustecho -n "${layoutLayerSizes[i]}"
        done
        mustecho ""
    fi
}

#----------------- Function ----------------
#name: get_executable(string)
#purpose: find the executable in an argument string
#-------------------------------------------
get_executable()
{
local i
for i in $1;
do
    if [[ $i != -* && -f $i && -x $i ]]
    then
        echo $i
        return 0
    fi
done
return 1
}

#----------------- Function ----------------
#name: run
#purpose: start the run mode
#-------------------------------------------
run()
{

mpiexec_command=${mpiexec_arg}

if [ ! -d $temp_arg ]
then
    musterror "temp directory not found, run prepare mode first!"
    exit 1
fi

if [ -f $temp_arg/mustrun.info ]
then
  totalSize=`cat $temp_arg/mustrun.info`
else
  musterror "Error: the \"mustrun.info\" file could not be found in the temp directory \"$temp_arg\", did you really prepare with this temporary directory?"
  exit 1
fi

if ! executable=$(get_executable "$input" )
then 
  musterror "Missing program to execute (parameter COMMAND, see '$0 --help' for more information)"
  exit 1
fi

if [ -z "$switchToUse" ] && ! get_num_procs
then
    if [ "@MPIEXEC_NUMPROC_FLAG@" ]
    then
        switchToUse="@MPIEXEC_NUMPROC_FLAG@"
    else
        switchToUse="-np"
    fi
fi
#####################################################################
## 1) Which programming language the program is using (no --must:language given)

myNM=@CMAKE_NM@
if [ -z "$language_arg" ]
then
    if [ -n "$myNM" -a myNM=$(which $myNM)  -o myNM=$(which nm) ]
    then
        if test -z "$($myNM $executable | grep MAIN__)"
        then
            language_arg="c"
        else
            language_arg="fortran"
        fi
    fi
fi

# #####################################################################
# ## 2) Do we want to ld_preload the pnmpi-lib?
myldd=@LDD@
myotool=@OTOOL@
havepnmpi=0

if [ $do_pnmpi_linked -ne 1 ]
then
    if [ $do_quiet -ne 1 ]
    then
        mustecho -n "Search for preloaded P^nMPI ... "
    fi
    if myldd=$(which $myldd)
    then
        if test -z "$($myldd $executable | grep pnmpi)"
        then
            if [ $do_quiet -ne 1 ]
            then
                mustecho -n "not found ... "
            fi
        else
            havepnmpi=1
            if [ $do_quiet -ne 1 ]
            then
                mustecho -n "found ... "
            fi
        fi
    elif myotool=$(which $myotool)
    then
        if test -z "$($myotool -L $executable | grep pnmpi)"
        then
            if [ $do_quiet -ne 1 ]
            then
                mustecho -n "not found ... "
            fi
        else
            havepnmpi=1
            if [ $do_quiet -ne 1 ]
            then
                mustecho -n "found ... "
            fi
        fi
    fi
else
    havepnmpi=1
fi

#additional options for valgrind
massifopt=""
memcheckopt=""
#memcheckopt="--log-file=valgrind/memcheck.out.%p --leak-check=full --track-origins=yes"

# insert pnmpize if needed
if test ${havepnmpi} -eq 0
then
    temp=""
    found=0
    for i in $input;
    do
        # already found the executable?
        if test $found -eq 0
        then
            if [ "$i" -ef "$executable" ]
            then
                absolut_path executable
                if [ $do_massif -eq 1 ]
                then
                    # use 'valgrind' with massif tool
                    executable="valgrind --tool=massif $massifopt $executable"
                fi
                if [ $do_memcheck -eq 1 ]
                then
                    # use 'valgrind' with memcheck tool
                    executable="valgrind --tool=memcheck $memcheckopt $executable"
                fi
                if [ $do_ddt -eq 1 ]
                then
                    # use ddt-client
                    if ! executable="$(which $precmdddt_arg) $executable"
                    then
                    	echo "$precmdddt_arg not found in PATH, please load ddt environment"
                    	exit 1
                    fi
                fi
                temp="${temp} @BASH@ @CMAKE_INSTALL_PREFIX@/bin/pnmpize $language_arg $executable"
                found=1
                continue
            fi
        fi
        temp="${temp} $i"
    done
    input=${temp}
else
    if [ $do_massif -eq 1 ]
    then
        # use 'valgrind' with massif tool
        mpiexec_command="$mpiexec_command valgrind --tool=massif $massifopt"
    fi
    if [ $do_memcheck -eq 1 ]
    then
        # use 'valgrind' with memcheck tool
        mpiexec_command="$mpiexec_command valgrind --tool=memcheck $memcheckopt"
    fi
    if [ $do_ddt -eq 1 ]
    then
        # use ddt-client
        if ! executable="$(which $precmdddt_arg) $executable"
        then
            echo "$precmdddt_arg not found in PATH, please load ddt environment"
            exit 1
        fi
    fi
fi

if [ $do_pnmpi_linked -ne 1 -a $do_quiet -ne 1 ]
then
    mustecho "success"
fi

#####################################################################
## 3) Run (finally :D)

if [ $do_timing -eq 1 ]
then
    # use 'time' to get the runtime of mpiexec
    mpiexec_command="time $mpiexec_command"
fi

export THIS_RUN_SEED=$$

if [ $do_verbose -eq 1 ]
then
  mustecho "========= Step 3) Execution ========="
else
  if [ $do_quiet -ne 1 -a "$mode_arg" != "run" ]
  then
    mustecho "Executing application:"
  fi
  
  #Tell P^nMPI to be quiet
  export PNMPI_BE_SILENT=1
fi

set +e #we ignore the final exit status, this simplifies regular expressions for tests

STACKWALKLIB=""
if [ -n "@CALLPATH_STACKWALKER_LIB_PATH@" ]
then
    LD_LIBRARY_PATH=@CALLPATH_STACKWALKER_LIB_PATH@:$LD_LIBRARY_PATH
    DYLD_LIBRARY_PATH=@CALLPATH_STACKWALKER_LIB_PATH@:$DYLD_LIBRARY_PATH
fi

# export PNMPI_LIB_PATH=$temp_arg/module_temp
export PNMPI_LIB_PATH=$temp_arg/BUILD/install/modules/:@CMAKE_INSTALL_PREFIX@/modules/:@GTI_MODULE_DIR@/
export LD_LIBRARY_PATH=$temp_arg/module_temp:@PnMPI_LIBRARY_DIR@:$LD_LIBRARY_PATH
export DYLD_LIBRARY_PATH=$temp_arg/module_temp:@PnMPI_LIBRARY_DIR@:$DYLD_LIBRARY_PATH
export PNMPI_CONF=$temp_arg/pnmpi.conf

#Remove the given number of tasks
  
input=$(echo "$input" | sed -e "s/$switchToUse[[:space:]]\([^[:space:]]*\)//")
mpiexec_arg="$mpiexec_arg $switchToUse $totalSize"
mpiexec_command="$mpiexec_command $switchToUse $totalSize"

input="@MPIEXEC_PREFLAGS@ $input @MPIEXEC_POSTFLAGS@"

if test $timeout_arg -ne 0
then

    #Verbose
    if [ $do_verbose -eq 1 ]
    then
      mustecho "Going to execute run with:"
      mustecho "bash -c \"@CMAKE_INSTALL_PREFIX@/bin/timeout -t $timeout_arg $mpiexec_command $input\""
    fi

    bash -c "@CMAKE_INSTALL_PREFIX@/bin/timeout -t $timeout_arg $mpiexec_command $input"

else

    #Verbose
    if [ $do_verbose -eq 1 ]
    then
      mustecho "Going to execute run with:"
      mustecho "$mpiexec_command $input"
    fi

    $mpiexec_command $input

fi

if [ $do_cleanshm -eq 1 ]
then
    set +e
    $mpiexec_arg @CMAKE_INSTALL_PREFIX@/bin/cleanup_shm >/dev/null 2>&1
    set -e
fi

if [ $do_clean -eq 1 -a ! "$temp_arg" -ef "$PWD" ]
    then
    if [ $do_verbose -eq 1 ]
        then
        mustecho "Information: remove intermediate data in directory \""$temp_arg"\"!"
    fi
    rm -fr "$temp_arg";
fi

if [ $do_quiet -ne 1 -a $output_arg == "html" -a $do_ddt -eq 0 ]
    then
    if [ -f ${PWD}/MUST_Output.html ]
	then
		mustecho "Execution finished, inspect \""${PWD}"/MUST_Output.html\"!"
    else
    	musterror "Execution finished, but no output found!"
    fi
elif [ $do_quiet -ne 1 -a $output_arg == "stdout" ]
    then
    mustecho "Execution finished."
fi
exit 0;

} # end run()

#Exit if something goes wrong
set -e

#####################################################################
## 1) clean up the input (replace any sequence of (white)spaces [:space:] by a single space,
##    remove leading, tailing spaces)
input=`echo "$*" | sed -e 's/[[:space:]]\{1,\}/ /g' -e 's/^[[:space:]]*//g' -e 's/[[:space:]]*$//g'`

oldenv=$(env)

#####################################################################
## 2) Option parsing
##

if opt="$(echo " $input " | sed -e 's/.*[[:space:]]\(-h\)[[:space:]].*/\1/')" && test "$opt" != " $input "
then
    printHelp
elif opt="$(echo " $input " | sed -e 's/.*[[:space:]]\(--help\)[[:space:]].*/\1/')" && test "$opt" != " $input "
then
    printHelp
elif opt="$(echo " $input " | sed -e 's/.*[[:space:]]\(-help\)[[:space:]].*/\1/')" && test "$opt" != " $input "
then
    printHelp
fi

for i in $(env | grep ^MUST | grep -v MUST_WAIT_AT_STARTUP | grep -v MUST_ROOT | grep -v MUST_INC | grep -v MUST_LIB); do 
    i=${i#MUST_}
    opt=${i%%=*}
    opt=$(echo $opt | tr A-Z a-z)
    case $opt in
    verbose | quiet | timing | massif | memcheck | nocrash | clean | cleanshm | pnmpi-linked | unused | info | distributed | dl | fillnodes | isp | scorep | ddt | reproduce | nodl | capture )
        envvars="${i%%=*} $envvars"
#       replace '-' by '_' in var names
        eval "do_${opt//-/_}=1"
        ;;

    # options with argument:
    mode | layout | np | mpiexec | temp | analyses | apis | language | nodesize | timeout | output | fanin | firstfanin | precmdddt )
        envvars="${i%%=*} $envvars"
        arg=${i#*=}
        # optionspecific tests:
        check_opt_arg $opt $arg
        eval "${opt//-/_}_arg=$arg"
        ;;
    *)
        mustecho "Unknown environment option MUST_$i is ignored"
    esac
done

while opt=$(echo " $input" | sed -e 's/.*[[:space:]]--must:\([^[:space:]]\{1,\}\).*/\1/') && test "$opt" != " $input" ;do
    case $opt in
    verbose | quiet | timing | massif | memcheck | nocrash | clean | cleanshm | pnmpi-linked | unused | info | distributed | dl | fillnodes | isp | scorep | ddt | reproduce | nodl | capture )
#       replace '-' by '_' in var names
        eval "do_${opt//-/_}=1"
        remove_single_opt $opt
        ;;

    # options with argument:
    mode | layout | np | mpiexec | temp | analyses | apis | language | nodesize | timeout | output | fanin | firstfanin | precmdddt )
        arg="$(get_opt_arg $opt)"
        # optionspecific tests:
        check_opt_arg $opt $arg
        remove_opt_and_arg $opt
        eval "${opt//-/_}_arg=$arg"
        ;;
    help)
        printHelp
        ;;
    version)
        printVersions
        exit 1
        ;;
    *)
        printHelp "Unknown option --must:$opt"
    esac
done

#####################################################################
## 2a) Scan for unused Options
if [ ${do_unused:=0} -eq 1 ]
then
    case ${mode_arg:=auto} in
    run)
        for i in nocrash layout analyses apis nodesize output
        do
            if eval "temp=\"\${do_$i}\${${i}_arg}\""; [ "$temp" ]
            then
                mustecho "Unused Option --must:$i"
            fi
        done
        ;;
    prepare)
        for i in timing massif memcheck clean cleanshm pnmpi-linked language timeout
        do
            if eval "temp=\"\${do_$i}\${$i_arg}\""; [ "$temp" ]
            then
                mustecho "Unused Option --must:$i"
            fi
        done
        ;;
    esac
fi


#####################################################################
## 3) Set default values

temp=${do_quiet:=0}
temp=${do_verbose:=0}
temp=${do_timing:=0}
temp=${do_massif:=0}
temp=${do_memcheck:=0}
temp=${do_nocrash:=0}
temp=${do_clean:=0}
temp=${do_fillnodes:=0}
temp=${do_distributed:=0}
temp=${do_dl:=1}
temp=${do_nodl:=0}
temp=${do_info:=0}
temp=${do_isp:=0}
temp=${do_ddt:=0}
temp=${do_reproduce:=0}
temp=${do_capture:=0}
temp=${do_scorep:=0}
temp=${do_cleanshm:=0}
temp=${do_pnmpi_linked:=0}
temp=${temp_arg:=must_temp}
temp=${mode_arg:=auto}
# temp=${layout_arg:=}
temp=${np_arg:="-np -n -p @MPIEXEC_NUMPROC_FLAG@"}
temp=${mpiexec_arg:="@MPIEXEC@"}
temp=${analyses_arg:=""}
# temp=${apis_arg:=}
# temp=${language_arg:=}
#temp=${nodesize_arg:=""}
temp=${timeout_arg:="0"}
temp=${output_arg:="html"}
temp=${fanin_arg:=16}
temp=${firstfanin_arg:=$fanin_arg}}
temp=${precmdddt_arg:="ddt-client"}

if [ $do_nodl -eq 1 ]
then
    do_dl=0
fi

workdir=$PWD

if [ $do_quiet -eq 0 -a -n "$envvars" ]
then
    mustecho "Using environment variable(s) $envvars"
fi
#####################################################################
## 3b) Make paths absolut

absolut_path temp_arg
analyses_arg=${analyses_arg//:/ }
analyses=""
for i in $analyses_arg; do
    absolut_path i
    analyses="$analyses $i"
done
apis_arg=${apis_arg//:/ }
apis=""
for i in $apis_arg; do
    absolut_path i
    apis="$apis $i"
done

if [ $do_info -eq 1 ]
then
    calclayers
	
    if [ $do_fillnodes -eq 1 ] 
	then 
		fillnodes
	fi
	
    print_info
	
    exit 0
fi

#####################################################################
## 3c) Auto mode

if [ ${mode_arg} == "auto" ]
then
    if [ ! -d ${temp_arg} -o ! -f ${temp_arg}/lastrun ]
    then
        mode_arg="preparerun"
    else
        calclayers
		
    if [ $do_fillnodes -eq 1 ] 
		then 
			fillnodes
		fi
		
        if get_checksum | diff - ${temp_arg}/lastrun 2>/dev/null 1>&2 
        then 
            mode_arg="run"
        else
            mode_arg="preparerun"
        fi
    fi
fi



#####################################################################
## 4) if prepare mode, run prepare()

case $mode_arg in prepare*)
    calclayers
	
	if [ $do_fillnodes -eq 1 ] 
	then 
		fillnodes
	fi
	
    if [ $do_quiet -eq 0 ]
    then
        print_info
    fi
	
    prepare
	;;
esac

#####################################################################
## 5) if run mode, run run()

case $mode_arg in *run)
    run;;
esac



